
  member

  include('QueueManager.inc'),once
  compile('_clarion_sync_object_', _C60_)
  include ('cwsynchm.inc'),once
  _clarion_sync_object_

  map
  end

QueueManagerClass.Construct           procedure

  code
  self.InitCriticalSection()
  self.SyncThreadQueue = false
  self.lFields = 0

QueueManagerClass.Destruct            procedure

  code
  if Not self.ManagedQueue &= NULL
    self.Free()
    if self.bDispose
      Dispose(self.ManagedQueue)
    end
  end
  self.KillCriticalSection()

QueueManagerClass.InitCriticalSection   procedure()
  code
  compile('_clarion_sync_object_', _C60_)
  self.QueueCS &= NewCriticalSection()
  _clarion_sync_object_

QueueManagerClass.KillCriticalSection   procedure()
  code
  compile('_clarion_sync_object_', _C60_)
  self.QueueCS.Kill()
  _clarion_sync_object_

QueueManagerClass.Lock                  procedure()
  code
  compile('_clarion_sync_object_', _C60_)
  self.QueueCS.Wait()
  _clarion_sync_object_

QueueManagerClass.Unlock                procedure()
  code
  compile('_clarion_sync_object_', _C60_)
  self.QueueCS.Release()
  _clarion_sync_object_

QueueManagerClass.Init                  procedure

result byte(false)
  code
  if Not self.ManagedQueue &= NULL
    result = true
  end
  return result

QueueManagerClass.Init PROCEDURE(*Queue q, byte bDispose=true)

  code
  self.Lock()
  self.ManagedQueue &= q
  self.bDispose = bDispose
  self.Unlock()

QueueManagerClass.Records PROCEDURE

result long(0)

  code
  self.Lock()
  if self.Init()
    result = RECORDS(self.ManagedQueue)
  end
  self.Unlock()
  return result

QueueManagerClass.Insert                procedure()

  code
  if self.Init()
    ADD(self.ManagedQueue)
  end

QueueManagerClass.Insert                procedure(Unsigned nRow)

  code
  if self.Init()
    ADD(self.ManagedQueue, nRow)
  end

QueueManagerClass.Insert PROCEDURE(*Queue q)

  code
  self.Lock()
  if self.Init()
    self.ManagedQueue = q
    ADD(self.ManagedQueue)
    if self.SyncThreadQueue
      Add(q)
    end
  end
  self.Unlock()

QueueManagerClass.Insert PROCEDURE(*Queue q, UNSIGNED nRow)

  code
  self.Lock()
  if self.Init()
    self.ManagedQueue = q
    ADD(self.ManagedQueue, nRow)
    if self.SyncThreadQueue
      Add(q, nRow)
    end
  end
  self.Unlock()

QueueManagerClass.Fetch PROCEDURE(UNSIGNED nRow)

  code
  if self.Init()
    GET(self.ManagedQueue, nRow)
  end

QueueManagerClass.Fetch PROCEDURE(*Queue q, UNSIGNED nRow)

  code
  self.Lock()
  clear(q)
  if self.Init()
    GET(self.ManagedQueue, nRow)
    q = self.ManagedQueue
  end
  self.Unlock()

QueueManagerClass.Update                procedure()

  code
  if self.Init()
    PUT(self.ManagedQueue)
  end

QueueManagerClass.Update PROCEDURE(*Queue q)

  code
  self.Lock()
  if self.Init()
    self.ManagedQueue = q
    PUT(self.ManagedQueue)
    if self.SyncThreadQueue
      Put(q)
    end
  end
  self.Unlock()

QueueManagerClass.Delete                procedure()

  code
  if self.Init()
    DELETE(self.ManagedQueue)
  end

QueueManagerClass.Delete PROCEDURE(*Queue q)

  code
  self.Lock()
  if self.Init()
    self.ManagedQueue = q
    DELETE(self.ManagedQueue)
    if self.SyncThreadQueue
      DELETE(q)
    end
  end
  self.Unlock()

QueueManagerClass.ClearBuffer                 procedure

  code
  if self.Init()
    CLEAR(self.ManagedQueue)
  end

QueueManagerClass.Free PROCEDURE

  code
  self.Lock()
  if self.Init()
    FREE(self.ManagedQueue)
    CLEAR(self.ManagedQueue)
  end
  self.Unlock()


QueueManagerClass.GetFieldIndex         procedure(String FieldName)

sField  astring
nFields long
result  long
i long

  code
  if self.Init()
    result = QMError:InvalidField
    nFields = self.Fields()
    sField = Upper(FieldName)
    i=1
    loop
      if self.who(i) = sField
        result = i
        i = nFields
      end
      i += 1
    while i <= nFields
  end
  return result

QueueManagerClass.Set                   procedure(String Field, ? QueueData)

FieldRef Any
nCol long

  code
  nCol = 0
  if self.Init()
    if Numeric(Field)
      nCol = Field
    else
      nCol = self.GetFieldIndex(Field)
    end

    if nCol > 0
      FieldRef &= what(self.ManagedQueue, nCol)
      FieldRef = QueueData
    end
  end

QueueManagerClass.SortQueue procedure(? k1)

  code
  self.Lock()
  if self.Init()
    Sort(self.ManagedQueue, k1)
  end
  self.Unlock()

QueueManagerClass.What PROCEDURE(UNSIGNED nCol)

qtype Any

  code
  if self.Init()
    qtype = WHAT(self.ManagedQueue, nCol)
  end
  return qtype

QueueManagerClass.Who PROCEDURE(UNSIGNED nCol)

sField astring

  code
  if self.Init()
    sField = WHO(self.ManagedQueue, nCol)
  end
  return sField

QueueManagerClass.Fields               procedure()

sField astring
i long

  code
  if self.lFields = 0
    i = 0
    loop
      i += 1
      sField = who(self.ManagedQueue, i)
    while Len(Clip(sField)) <> 0
    self.lFields = i - 1
  end
  return self.lFields

QueueManagerClass.GetQueue	procedure(*Queue q)

nRecords long
sField astring
FieldRef Any
i long
j long

  code
  self.Lock()
  if self.Init()
    nRecords = records(self.ManagedQueue)
    free(q)
    loop i=1 to nRecords
      Clear(self.ManagedQueue)
      Get(self.ManagedQueue, i)
      ! would like to use q :=: self.ManagedQueue but it doesn't work in version 6 generically
      j = 1
      loop
        sField = who(q, j)
        if Len(Clip(sField)) <> 0
          FieldRef &= what(q, j)
          FieldRef = what(self.ManagedQueue, j)
        else
          break
        end
        j += 1
      end
      Add(q)
    end
  end
  self.Unlock()

